@@ -11,10 +11,11 @@ __author__ = 'Martin Pitt'
 __email__ = 'martin.pitt@ubuntu.com'
 __copyright__ = '(c) 2012 Canonical Ltd.'
 __license__ = 'LGPL 3+'
-__version__ = '0.8.1'
+__version__ = '0.9'
 
-from dbusmock.mockobject import DBusMockObject, MOCK_IFACE, get_object, get_objects
+from dbusmock.mockobject import (DBusMockObject, MOCK_IFACE,
+                                 OBJECT_MANAGER_IFACE, get_object, get_objects)
 from dbusmock.testcase import DBusTestCase
 
-__all__ = ['DBusMockObject', 'MOCK_IFACE', 'DBusTestCase', 'get_object',
-           'get_objects']
+__all__ = ['DBusMockObject', 'MOCK_IFACE', 'OBJECT_MANAGER_IFACE',
+           'DBusTestCase', 'get_object', 'get_objects']
