@@ -15,6 +15,7 @@ import unittest
 import sys
 import subprocess
 import time
+import os
 
 import dbus
 
@@ -27,11 +28,14 @@ p = subprocess.Popen(['which', 'upower'], stdout=subprocess.PIPE)
 p.communicate()
 have_upower = (p.returncode == 0)
 
-p = subprocess.Popen(['upower', '--version'], stdout=subprocess.PIPE,
-                     universal_newlines=True)
-out = p.communicate()[0]
-upower_client_version = out.splitlines()[0].split()[-1]
-assert p.returncode == 0
+if have_upower:
+    p = subprocess.Popen(['upower', '--version'], stdout=subprocess.PIPE,
+                         universal_newlines=True)
+    out = p.communicate()[0]
+    upower_client_version = out.splitlines()[0].split()[-1]
+    assert p.returncode == 0
+else:
+    upower_client_version = 0
 
 
 @unittest.skipUnless(have_upower, 'upower not installed')
@@ -243,8 +247,15 @@ class TestUPower1(dbusmock.DBusTestCase):
         self.assertEqual(props['Percentage'], 50.0)
         self.assertEqual(props['WarningLevel'], 3)
 
+        env = os.environ.copy()
+        env['LC_ALL'] = 'C'
+        try:
+            del env['LANGUAGE']
+        except KeyError:
+            pass
+
         out = subprocess.check_output(['upower', '--dump'],
-                                      universal_newlines=True)
+                                      universal_newlines=True, env=env)
         self.assertIn('/DisplayDevice\n', out)
         self.assertIn('  battery\n', out)  # type
         self.assertRegex(out, 'state:\s+charging')
@@ -252,8 +263,8 @@ class TestUPower1(dbusmock.DBusTestCase):
         self.assertRegex(out, 'energy:\s+40 Wh')
         self.assertRegex(out, 'energy-full:\s+80 Wh')
         self.assertRegex(out, 'energy-rate:\s+2.5 W')
-        self.assertRegex(out, 'time to empty:\s+1,0 hours')
-        self.assertRegex(out, 'time to full:\s+30,0 minutes')
+        self.assertRegex(out, 'time to empty:\s+1\.0 hours')
+        self.assertRegex(out, 'time to full:\s+30\.0 minutes')
         self.assertRegex(out, 'present:\s+yes')
         self.assertRegex(out, "icon-name:\s+'half-battery'")
         self.assertRegex(out, 'warning-level:\s+low')
